#include "unlitmaterial.h"

using namespace std;

namespace Msp {
namespace GL {

const Tag UnlitMaterial::texture_tags[] =
{
	Tag("color_tex"),
	Tag()
};

UnlitMaterial::UnlitMaterial():
	texture(0),
	vertex_color(false)
{
	set_color(Color(1.0f));
}

void UnlitMaterial::fill_program_info(string &module_name, map<string, int> &spec_values) const
{
	module_name = "unlit.glsl";
	spec_values["use_texture"] = (texture!=0);
	spec_values["use_vertex_color"] = vertex_color;
}

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
void UnlitMaterial::attach_textures_to(Texturing &texturing, ProgramData &tex_shdata) const
{
	attach_texture_to(texture, texturing, tex_shdata, "color_tex");
}
#pragma GCC diagnostic pop

const Texture *UnlitMaterial::get_texture(Tag tag) const
{
	if(tag==texture_tags[0])
		return texture;
	else
		return 0;
}

void UnlitMaterial::set_texture(const Texture *tex)
{
	texture = tex;
}

void UnlitMaterial::set_color(const Color &c)
{
	color = c;
	shdata.uniform("unlit_material.color", color);
}

void UnlitMaterial::set_vertex_color(bool vc)
{
	vertex_color = vc;
}


DataFile::Loader::ActionMap UnlitMaterial::Loader::shared_actions;

UnlitMaterial::Loader::Loader(UnlitMaterial &m):
	DerivedObjectLoader<UnlitMaterial, Material::PropertyLoader<UnlitMaterial> >(m)
{
	set_actions(shared_actions);
}

UnlitMaterial::Loader::Loader(UnlitMaterial &m, Collection &c):
	DerivedObjectLoader<UnlitMaterial, Material::PropertyLoader<UnlitMaterial> >(m, c)
{
	set_actions(shared_actions);
}

void UnlitMaterial::Loader::init_actions()
{
	Material::PropertyLoader<UnlitMaterial>::init_actions();
	add("texture", &Loader::property_texture, &UnlitMaterial::set_texture);
	add_property("color", &UnlitMaterial::set_color, 0, true);
	add("vertex_color", &UnlitMaterial::vertex_color);
}

} // namespace GL
} // namespace Msp
