#include <algorithm>
#include "orderedscene.h"
#include "renderer.h"

namespace Msp {
namespace GL {

void OrderedScene::add(Renderable &r)
{
	renderables.push_back(&r);
}

void OrderedScene::remove(Renderable &r)
{
	RenderableList::iterator end = std::remove(renderables.begin(), renderables.end(), &r);
	renderables.erase(end, renderables.end());
}

void OrderedScene::prepend(Renderable &r)
{
	renderables.push_front(&r);
}

void OrderedScene::insert(unsigned index, Renderable &r)
{
	RenderableList::iterator i = renderables.begin();
	for(; (i!=renderables.end() && index); ++i, --index) ;
	renderables.insert(i, &r);
}

void OrderedScene::insert_after(Renderable &after, Renderable &r)
{
	RenderableList::iterator i = renderables.begin();
	for(; (i!=renderables.end() && *i!=&after); ++i) ;
	renderables.insert(i, &r);
}

void OrderedScene::setup_frame(Renderer &renderer)
{
	for(RenderableList::const_iterator i=renderables.begin(); i!=renderables.end(); ++i)
		(*i)->setup_frame(renderer);
}

void OrderedScene::finish_frame()
{
	for(RenderableList::const_iterator i=renderables.begin(); i!=renderables.end(); ++i)
		(*i)->finish_frame();
}

void OrderedScene::render(Renderer &renderer, Tag tag) const
{
	if(setup_frustum(renderer))
	{
		for(RenderableList::const_iterator i=renderables.begin(); i!=renderables.end(); ++i)
			if(!frustum_cull(**i))
				renderer.render(**i, tag);
	}
	else
	{
		for(RenderableList::const_iterator i=renderables.begin(); i!=renderables.end(); ++i)
			renderer.render(**i, tag);
	}
}

} // namespace GL
} // namespace Msp
